CASUALTY = {}
--CASUALTY.Timeout = nil
CASUALTY.casualties = {}
CASUALTY_CONFIG = {
   ["Delay"] = 1,
   ["SpeechDelay"] = 2,
   ["Enabled"] = true,
   ["CheckPartyRaidOnly"] = true,
   ["CheckDestroyedObjects"] = true,
   ["CheckPets"] = true
}

function CASUALTY.OnLoad()
   this:RegisterEvent("CHAT_MSG_COMBAT_FRIENDLY_DEATH")
   SLASH_CASUALTY1 = "/casualty"
   SlashCmdList["CASUALTY"]=CASUALTY.ConsoleCommand
end

function CASUALTY.ToggleEnabled()
   CASUALTY_CONFIG.Enabled = CASUALTY_CONFIG.Enabled == false
   if CASUALTY_CONFIG.Enabled then
      yak_print("Casualty reporting enabled")
   else
      yak_print("Casualty reporting disabled")
      CASUALTY.casualties = {}
   end
end

function CASUALTY.Config()
   CasualtyConfig:Show()
end

CASUALTY.ConsoleCommands = {
   ["toggle"] = CASUALTY.ToggleEnabled,
   ["config"] = CASUALTY.Config
}

function CASUALTY.ConsoleCommand(message)
   local command
   local args
   command, args = yak_console_command_string(message)
--   yak_print(command)
   comfunc = CASUALTY.ConsoleCommands[command]
   if comfunc ~= nil then
      comfunc(args)
   else
      yak_print("/casualty         -- gives you this usage statement")
      yak_print("/casualty config  -- opens config dialog")
      yak_print("/casualty toggle  -- toggles casualty reporting")
   end
end

function CASUALTY.matches_message( arg, template )
   -- returns whether or not the string matches the template
   -- and as a second return, the name of the object matched
   local msg = string.gsub(template, "%%s", "(.-)")
   -- following gyrations are to match the empty message, particularly with
   -- PARTYKILLOTHER
   local emptymsg = string.gsub(template, "%%s", "",1)
   emptymsg = string.gsub(emptymsg, "%%s", "(.-)")
   local beginpos, endpos = string.find(arg,msg)
   if beginpos then
--      yak_print(emptymsg)
      nbegin, nend = string.find(arg, emptymsg)
--      yak_print("arg: "..arg.."; beginpos: "..nbegin.."; name: "..string.sub(arg,1,nbegin-1))
      return true, string.sub(arg, 1, nbegin-1)
   else
      return false
   end
end

function CASUALTY.log_death_if_matched(arg, template)
   local matched, name = CASUALTY.matches_message(arg, template)
--   if matched then yak_print("matched "..name) else yak_print("not matched") end
   if matched and CASUALTY.IsReportableCasualty(name) then
--      yak_print("adding casualty: ".. name)
--      if CASUALTY.Timeout then
      if table.getn(CASUALTY.casualties) == 0 then
	 YakScheduler:In(CASUALTY_CONFIG.Delay, "CASUALTY.Report", CASUALTY.Report)
      end
      table.insert( CASUALTY.casualties, name )
--      else
--	 CASUALTY.Timeout = GetTime() + CASUALTY_CONFIG.Delay
--	 CASUALTY.casualties = {name}
--      end
   end
end

function CASUALTY.NamedUnitIsPet( name )
   local result = false
   TargetByName( name )
   if UnitCreatureType("target") == "Beast" then
      result = true
   end
   TargetLastTarget()
   return result
end

function CASUALTY.NamedUnitIsInPartyOrRaid( name )
   local result = false
   if name == nil then return false end
   -- got a strange usage question where this was called with some weird name
   -- yak_print("Checking unit named "..name)
   TargetByName(name)
   if yak_PlayerIsInRaid() then
      result = UnitInRaid( "target" )
   elseif yak_PlayerIsInGroup() then
      result = UnitInParty("target")
   end
   TargetLastTarget()
   return result
end

function CASUALTY.IsReportableCasualty(name)
   -- note: objects (totems) are checked earlier
   local result = true
   if CASUALTY_CONFIG.CheckPartyRaidOnly then
      result = result and CASUALTY.NamedUnitIsInPartyOrRaid()
   elseif not CASUALTY_CONFIG.CheckPets then
      result = result and (not CASUALTY.NamedUnitIsPet( name ))
   end
   return result
end

function CASUALTY.on_death_event()
   if CASUALTY_CONFIG.Enabled then
      -- look at the argument to see if it's something we want to match
      CASUALTY.log_death_if_matched(arg1, PARTYKILLOTHER)
      if CASUALTY_CONFIG.CheckDestroyedObjects then
--	 yak_print("Checking destroyed object")
	 CASUALTY.log_death_if_matched(arg1, UNITDESTROYEDOTHER)
      end
      CASUALTY.log_death_if_matched(arg1, UNITDIESOTHER)
      --   CASUALTY.log_death_if_matched(arg, UNITDIESSELF)
   end
end

CASUALTY.EventHandlers = {
   ["CHAT_MSG_COMBAT_FRIENDLY_DEATH"] = CASUALTY.on_death_event
}

function CASUALTY.OnEvent(event)
   func = CASUALTY.EventHandlers[event]
   if func then
      func()
   end
end

--function CASUALTY.OnUpdate()
--   if CASUALTY.Timeout and CASUALTY_CONFIG.Enabled then
--      if CASUALTY.Timeout < GetTime() then
--	 -- we've timed out since seeing a death; time to report our message
--	 if table.getn( CASUALTY.casualties ) == 1 then
--	    CASUALTY.ReportCasualty()
--	 elseif table.getn( CASUALTY.casualties ) > 1 then
--	    CASUALTY.ReportCasualties()
--	 else
--	    CASUALTY.Timeout = nil
--	 end
--      end
--   end
--end

function CASUALTY.Report()
   if table.getn(CASUALTY.casualties) == 1 then
      CASUALTY.ReportCasualty()
   elseif table.getn(CASUALTY.casualties) > 1 then
      CASUALTY.ReportCasualty()
   end
end
   
function CASUALTY.ReportCasualty()
   DEFAULT_CHAT_FRAME:AddMessage(CASUALTY_STRINGS.Casualty .. ": " .. CASUALTY.casualties[1])
   PlaySoundFile(CASUALTY_SOUND_PATHS.Casualty)
   CASUALTY.casualties = {}
   YakScheduler:In(CASUALTY_CONFIG.SpeechDelay, "CASUALTY.Report", CASUALTY.Report)
--   CASUALTY.Timeout = GetTime() + CASUALTY_CONFIG.SpeechDelay
end

function CASUALTY.ReportCasualties()
   local msg = CASUALTY_STRINGS.Casualties .. ": "
   for k,v in ipairs(CASUALTY.casualties) do
      msg = msg .. v..", "
   end
   msg = string.sub(msg, 1, -3)
   DEFAULT_CHAT_FRAME:AddMessage(msg)
   PlaySoundFile(CASUALTY_SOUND_PATHS.Casualties)
   CASUALTY.casualties = {}
   YakScheduler:In(CASUALTY_CONFIG.SpeechDelay, "CASUALTY.Report", CASUALTY.Report)
--   CASUALTY.Timeout = GetTime() + CASUALTY_CONFIG.SpeechDelay
end

function CASUALTY.OnDragDelay()
   CASUALTY_CONFIG.Delay = this:GetValue()
   getglobal(this:GetName().."Text"):SetText(CASUALTY_CONFIG_STRINGS.DelaySlider .. ": "..this:GetValue().." s");
end

function CASUALTY.OnDragSpeechDelay()
   CASUALTY_CONFIG.SpeechDelay = this:GetValue()
   getglobal(this:GetName().."Text"):SetText(CASUALTY_CONFIG_STRINGS.DelaySlider .. ": "..this:GetValue().." s");
end